#ifndef NODE_HPP
#define NODE_HPP

#include <vector>

class Node {
public:
    // default constructor creates a leaf node
    Node() = default;
    // whoever creates the node is responsible for deciding which particles are in which octant and for allocating the localParticles array 
    // node will write to localParticles array
    Node(Node* root, Node* parent, double* masses, double* particles, unsigned nParticles, unsigned depth, 
        double size, double center[3], std::vector<unsigned>& localParticles, unsigned nLocalParticles);
    // destuctor
    ~Node();
    // copy constructor
    Node(const Node&) = default;
    // copy assignment
    Node& operator=(const Node&);
    // update tree: visit each node, check if it needs to be split or merged (for example if a particle has left the region)
    void update(std::vector<unsigned>& allParticles);
    // calculate force on a particle recursively
    double* calculateForce(unsigned particle);
private:
    Node* root_;
    Node* parent_;
    Node* children_[8];
    double* masses_;
    double* particles_;
    unsigned nParticles_;
    unsigned depth_;
    double size_;
    double center_[3];
    std::vector<unsigned> localParticles_;
    unsigned nLocalParticles_;
    double mass_;
    const double theta0_ = 0.4;
    bool isLeaf_;
};

#endif /* NODE_HPP */